package com.scbfkj.uni.api;

import com.scbfkj.uni.library.DataFormatUtil;
import com.scbfkj.uni.library.RequestUtil;
import com.scbfkj.uni.library.UniReturnUtil;
import com.scbfkj.uni.process.DataBase;
import com.scbfkj.uni.service.LoggerService;
import com.scbfkj.uni.service.SecurityService;
import com.scbfkj.uni.system.Config;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import java.time.LocalDateTime;
import java.util.*;

@Component
@Aspect
public class LogAop {


    @Resource
    private SecurityService securityService;

    private static List<Map<String, Object>> apiInfos;


    @Around(value = "within( com.scbfkj.uni.api.*Api)")
    public ResponseEntity<Map<String, Object>> invokeAround(ProceedingJoinPoint joinPoint) {
        LocalDateTime requestTime = LocalDateTime.now();

        ServletRequestAttributes requestAttributes = (ServletRequestAttributes) RequestContextHolder.currentRequestAttributes();
//            前端需求添加header
//        if (requestAttributes.getResponse() != null) {
//            requestAttributes.getResponse().addHeader("Set-Cookie", "SameSite=None");
//        }
        HttpServletRequest request = requestAttributes.getRequest();

//        请求
        String uri = request.getRequestURI();
//        请求参数
        Object[] args = joinPoint.getArgs();
        Object returnData = null;
        String message = null;

        try {

            if (Config.securityEnable) {
                if (Objects.isNull(apiInfos) || apiInfos.isEmpty()) {
                    apiInfos = DataBase.query(Config.securityConnectionStr, "select * from apiinfo", new ArrayList<>() {{
                        add(new Object[]{});
                    }});
                }

                Optional<Map<String, Object>> requestpath = apiInfos.stream().filter(it -> {
                    Object o = it.get("requestpath");
                    return Objects.equals(o, uri);
                }).findAny();
                if (requestpath.isPresent()) {
                    Map<String, Object> stringObjectMap = requestpath.get();
                    Object o = stringObjectMap.get("securitykey");
                    if (Objects.nonNull(o)) {
                        String[] securityCheck = o.toString().split(",");
                        for (String s : securityCheck) {
                            if (Objects.equals(s, "token")) {
                                String appToken = RequestUtil.getAppToken();
                                if (Objects.isNull(appToken)) {
                                    return ResponseEntity.ok(UniReturnUtil.fail("没有找到token"));
                                }
                                Map<String, Object> result = securityService.verifyToken(appToken);
                                if (!result.get("code").equals("0")) {
                                    return ResponseEntity.ok(result);
                                }
                            }
                            if (Objects.equals("usertoken", s)) {
                                String userToken = RequestUtil.getUserToken();
                                if (Objects.isNull(userToken)) {
                                    return ResponseEntity.ok(UniReturnUtil.fail("没有找到 usertoken"));
                                }
                                Map<String, Object> result = securityService.checkUserToken(userToken);
                                if (!result.get("code").equals("0")) {
                                    return ResponseEntity.ok(result);
                                }
                            }
                        }
                    }
                }
            }
            returnData = joinPoint.proceed(args);
            return ((ResponseEntity<Map<String, Object>>) returnData);
        } catch (Throwable e) {
            if (Config.debug) {
                e.printStackTrace();
            }
//            错误异常消息
            message = UniReturnUtil.getMessage(e);
            return ResponseEntity.ok(UniReturnUtil.fail(message));
        } finally {
            Object finalReturnData = returnData;
            LoggerService.log("interfacelog", new HashMap<>() {{
                put("requesttime", requestTime);
                put("requestpath", uri);
                put("requestdata", DataFormatUtil.toString(args));
                put("sessionid", RequestUtil.getSessionId());
                put("returndata", DataFormatUtil.toString(finalReturnData));
                try {
                    put("applicationid", RequestUtil.getAppId());
                } catch (Exception e) {
                    if (Config.debug) {
                        e.printStackTrace();
                    } else {
                        System.out.println(UniReturnUtil.getMessage(e));
                    }
                }
            }});
        }
    }
}
